@@ -1,6 +1,6 @@
 package Perl6::Pod::Parser;
 
-#$Id: Parser.pm 574 2009-07-13 03:57:29Z zag $
+#$Id$
 
 =pod
 
@@ -22,6 +22,7 @@ DOCUMENTING !DOCUMENTING !DOCUMENTING !DOCUMENTING !DOCUMENTING !
 use warnings;
 use strict;
 use Carp;
+use open ':utf8';
 use IO::File;
 use Test::More;
 use Data::Dumper;
@@ -39,7 +40,8 @@ sub new {
     unless ( exists $self->{__DEFAULT_CONTEXT} ) {
 
         #create default context
-        my $context = new Perl6::Pod::Parser::Context::;
+        my $context =
+          new Perl6::Pod::Parser::Context:: vars => { root => $self };
 
         #setup defaults
         $self->{__DEFAULT_CONTEXT} = $context;
@@ -69,12 +71,15 @@ sub mk_block {
       || 'Perl6::Pod::Block';
 
     #      or die "Unknown block_type $name. Try =use ...";
+    #create class_options
+    my $class_options = $self->current_context->class_opts->{$name};
 
     #get prop
     my $block = $mod_name->new(
-        name    => $name,
-        context => $self->current_context,
-        options => $pod_opt
+        name          => $name,
+        context       => $self->current_context,
+        options       => $pod_opt,
+        class_options => $class_options
     );
 
 }
@@ -89,16 +94,16 @@ sub mk_fcode {
 
     #make first element
     my ( $name, $pod_opt ) = @_;
-    my $mod_name = $self->current_context->use->{$name."<>"}
+    my $mod_name = $self->current_context->use->{ $name . "<>" }
       || 'Perl6::Pod::FormattingCode';
 
     #      or die "Unknown block_type $name. Try =use ...";
-
     #get prop
     my $block = $mod_name->new(
         name    => $name,
         context => $self->current_context,
-        options => $pod_opt
+        options => $pod_opt,
+        class_options => $self->current_context->class_opts->{ $name . "<>" }
     );
 
 }
@@ -151,6 +156,14 @@ sub on_start_element {
 sub on_start_block {
     my $self = shift;
     my $blk  = shift;
+
+    #    warn "start element ". $blk->local_name;
+    # call on_child for curretn element
+    if ( my $elem = $self->current_element ) {
+
+#    warn "on child!" .$elem->local_name ." -> on_child( ". $blk->local_name.")";;
+#        $elem->on_child( $blk)
+    }
     $blk->start( $self, $blk->get_attr() );
     return $blk;
 }
@@ -179,7 +192,7 @@ sub on_characters {
     return $self->on_para(@_);
 }
 
-############### 5 basic events 
+############### 5 basic events
 
 sub begin_input {
     my $self = shift;
@@ -191,24 +204,35 @@ sub end_input {
     $self->end_document;
 }
 
-
 sub start_block {
     my $self = shift;
-    my ( $name, $opt ,$str_num ) = @_;
-    my $elem = $self->mk_block( $name, $opt );
+    my ( $name, $opt, $str_num ) = @_;
+    my $elem = ref($name) ? $name : $self->mk_block( $name, $opt );
     $self->start_element($elem);
 }
 
+sub __expand_array_ref {
+    my $self = shift;
+    my @res  = ();
+    for (@_) {
+        push @res, ref($_) eq 'ARRAY' ? $self->__expand_array_ref(@$_) : $_;
+    }
+    @res;
+}
+
 sub para {
     my $self = shift;
     my $txt  = shift;
-
     #hadnle block on_para
     if ( my $elem = $self->current_element ) {
-        $txt = $elem->on_para($self, $txt);
+        $txt = $elem->on_para( $self, $txt );
+        if ( ref($txt) ) {
+            $self->run_para($txt);
+        }
+        else {
+            $self->_process_comm( $self->mk_characters($txt) );
+        }
     }
-    my $elems = $self->get_elements_from_ref( $self->parse_str($txt) );
-    $self->_process_comm($_) for @$elems;
 }
 
 sub end_block {
@@ -218,30 +242,74 @@ sub end_block {
     $self->end_element($elem);
 }
 
-sub _parse_tree_ {
+sub run_para {
     my $self = shift;
-    my $elem = shift;
-    if ( ref($elem) ) {
-        if ( UNIVERSAL::isa( $elem, 'Pod::ParseTree' ) ) {
-            return [ map { ref($_) ? $self->_parse_tree_($_) : $_ }
-                  $elem->children ];
+    my @in   = $self->__expand_array_ref(@_);
+    foreach my $el (@in) {
+        if ( UNIVERSAL::isa( $el, 'XML::ExtOn::Element' ) ) {
+            $self->_process_comm($el);
+            next;
         }
-        elsif ( UNIVERSAL::isa( $elem, 'Pod::InteriorSequence' ) ) {
-            my %attr = ( name => $elem->cmd_name );
-            if ( my $ptree = $elem->parse_tree ) {
-                $attr{childs} = $self->_parse_tree_($ptree);
+        unless ( exists $el->{type} ) {
+            $self->__process_events( $self->__make_events($el) );
+        }
+        else {
+            if ( $el->{type} eq 'para' ) {
+                $self->_process_comm( $self->mk_characters( $el->{data} ) );
             }
-            return \%attr;
         }
     }
 }
 
-sub parse_str {
+#make events for root parser
+sub __make_events {
     my $self = shift;
-    my $str  = shift;
-    my $p    = new Pod::Parser::;
-    my $res  = $self->_parse_tree_( Pod::Parser->new->parse_text( $str, 123 ) );
-    return $res;
+    my @res  = ();
+    foreach my $el (@_) {
+        unless ( ref($el) ) {
+            $el = { type => 'para', data => $el };
+        }
+
+        #process refs
+        if ( exists $el->{type} ) {
+            push @res, $el;
+        }
+        else {
+            my $name = $el->{name};
+
+            #make start stop
+            push @res,
+              {
+                type => 'start_fcode',
+                data => $name
+              },
+              $self->__make_events( ref($el->{childs}) ? @{ $el->{childs} } : $el->{childs}),
+              {
+                type => 'end_fcode',
+                data => $name
+              };
+        }
+    }
+    return @res;
+}
+
+sub __process_events {
+    my $self  = shift;
+    my $rootp = $self;    #->context->{vars}->{root};
+    foreach my $ev (@_) {
+        my $type = $ev->{type};
+        if ( $type eq 'para' ) {
+            $rootp->para( $ev->{data} );
+        }
+        else {
+
+            #process format code
+            my $fc = $self->mk_fcode( $ev->{data} );
+            ( $ev->{type} eq 'start_fcode' )
+              ? $rootp->start_block( $fc, '', 0 )
+              : $rootp->end_block( $fc, '', 0 );
+        }
+    }
 }
 
 sub get_elements_from_ref {
@@ -268,14 +336,28 @@ sub get_elements_from_ref {
     return \@elems;
 }
 
-
+#make XML::ExtOn objects from array
+sub _make_elements {
+    my $self = shift;
+    my @res  = ();
+    for (@_) {
+        push @res, ref($_)
+          ? ref($_) eq 'ARRAY'
+              ? $self->_make_elements(@$_)
+              : $_
+          : $self->mk_characters($_);
+    }
+    return @res;
+}
 1;
 __END__
 
 
 =head1 SEE ALSO
 
-L<http://perlcabal.org/syn/S26.html>
+L<http://zag.ru/perl6-pod/S26.html>,
+Perldoc Pod to HTML converter: L<http://zag.ru/perl6-pod/>,
+Perl6::Pod::Lib
 
 =head1 AUTHOR
 
@@ -283,7 +365,7 @@ Zahatski Aliaksandr, <zag@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2009 by Zahatski Aliaksandr
+Copyright (C) 2009-2010 by Zahatski Aliaksandr
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.8 or,
