@@ -4,8 +4,20 @@ use strict;
 use Perl6::Pod::Directive::use;
 use Perl6::Pod::Directive::config;
 use Perl6::Pod::Block::comment;
-
+use Perl6::Pod::Block::code;
+use Perl6::Pod::Block::pod;
+use Perl6::Pod::Block::para;
+use Perl6::Pod::Block::table;
+use Perl6::Pod::Block::output;
+use Perl6::Pod::Block::input;
+use Perl6::Pod::FormattingCode::C;
 use Perl6::Pod::FormattingCode::M;
+use Perl6::Pod::FormattingCode::L;
+use Perl6::Pod::FormattingCode::P;
+use Perl6::Pod::FormattingCode::B;
+use Perl6::Pod::FormattingCode::I;
+use Perl6::Pod::FormattingCode::X;
+use Perl6::Pod::FormattingCode::E;
 
 use Tie::UnionHash;
 use Data::Dumper;
@@ -22,10 +34,24 @@ my $OPTION_EXTRACT = qr{ :()($IDENT)($BALANCED_BRACKETS?) | :(!)($IDENT)() }xms;
 
 use constant {
     DEFAULT_USE => {
-        use    => 'Perl6::Pod::Directive::use',
-        config => 'Perl6::Pod::Directive::config',
+        use     => 'Perl6::Pod::Directive::use',
+        config  => 'Perl6::Pod::Directive::config',
         comment => 'Perl6::Pod::Block::comment',
-        'M<>' =>'Perl6::Pod::FormattingCode::M'
+        code    => 'Perl6::Pod::Block::code',
+        pod     => 'Perl6::Pod::Block::pod',
+        para    => 'Perl6::Pod::Block::para',
+        table   => 'Perl6::Pod::Block::table',
+        output  => 'Perl6::Pod::Block::output',
+        input   => 'Perl6::Pod::Block::input',
+        'C<>'   => 'Perl6::Pod::FormattingCode::C',
+        'M<>'   => 'Perl6::Pod::FormattingCode::M',
+        'L<>'   => 'Perl6::Pod::FormattingCode::L',
+        'B<>'   => 'Perl6::Pod::FormattingCode::B',
+        'I<>'   => 'Perl6::Pod::FormattingCode::I',
+        'X<>'   => 'Perl6::Pod::FormattingCode::X',
+
+        #        'P<>'   => 'Perl6::Pod::FormattingCode::P',
+        'E<>' => 'Perl6::Pod::FormattingCode::E',
     }
 };
 
@@ -39,10 +65,12 @@ sub new {
 
     #set default contexts
     my %args = (
-        _usef     => {},
-        _use      => DEFAULT_USE,
-        _config   => {},
-        _encoding => 'UTF-8',
+        _usef       => {},
+        _use        => DEFAULT_USE,
+        _config     => {},
+        _encoding   => 'UTF-8',
+        _custom     => {},
+        _class_opts => {},
         @_
     );
 
@@ -81,7 +109,7 @@ sub config {
 
 =head2 usef
 
-return ref to hash of pod options per blockname
+return ref to hash of plugin  per formatcode name
 
 =cut
 
@@ -91,7 +119,7 @@ sub usef {
 
 =head2 use
 
-return ref to hash of pod options per blockname
+return ref to hash of perl module per blockname
 
 =cut
 
@@ -99,6 +127,26 @@ sub use {
     return $_[0]->{_use};
 }
 
+=head2 custom
+
+return ref to hash of user defined keys,vals
+
+=cut
+
+sub custom {
+    return $_[0]->{_custom};
+}
+
+=head2 class_opts
+
+return ref to hash of Class optioons to create loaded by use mods
+
+=cut
+
+sub class_opts {
+    return $_[0]->{_class_opts};
+}
+
 =head2 set_use <module_name>, ':config_options'
 
 =cut
@@ -339,4 +387,25 @@ sub set_attr {
 }
 
 1;
+__END__
+
+=head1 SEE ALSO
+
+L<http://zag.ru/perl6-pod/S26.html>,
+Perldoc Pod to HTML converter: L<http://zag.ru/perl6-pod/>,
+Perl6::Pod::Lib
+
+=head1 AUTHOR
+
+Zahatski Aliaksandr, <zag@cpan.org>
+
+=head1 COPYRIGHT AND LICENSE
+
+Copyright (C) 2009-2010 by Zahatski Aliaksandr
+
+This library is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself, either Perl version 5.8.8 or,
+at your option, any later version of Perl 5 you may have available.
+
+=cut
 
