@@ -1,4 +1,4 @@
-package Test::Filter;
+package Test::FilterDocBook;
 use strict;
 use warnings;
 use Test::More;
@@ -16,21 +16,44 @@ sub on_start_element {
 
 package Perl6::Pod::To::DocBook;
 
-#$Id: DocBook.pm 574 2009-07-13 03:57:29Z zag $
+#$Id$
 
 =pod
 
 =head1 NAME
 
-Perl6::Pod::To::DocBook - DocBook formatter 
+Perl6::Pod::To::DocBook - DocBook formater 
 
 =head1 SYNOPSIS
 
-    header => 0, doctype => 'chapter'
+    my $p = new Perl6::Pod::To::DocBook:: 
+                header => 0, doctype => 'chapter';
+
 
 =head1 DESCRIPTION
 
-DOCUMENTING !DOCUMENTING !DOCUMENTING !DOCUMENTING !DOCUMENTING !
+Process pod to docbook
+
+Sample:
+
+        =begin pod
+        =NAME Test chapter
+        =para This is a test para
+        =end pod
+
+Run converter:
+
+        pod6docbook test.pod > test.xml
+
+Result xml:
+
+        <?xml version="1.0"?>
+        <chapter>
+          <title>Test chapter
+        </title>
+          <para>This is a test para
+        </para>
+        </chapter>
 
 
 =cut
@@ -38,9 +61,11 @@ DOCUMENTING !DOCUMENTING !DOCUMENTING !DOCUMENTING !DOCUMENTING !
 use strict;
 use warnings;
 use Perl6::Pod::To::XML;
-use XML::SAX::Writer;
 use Perl6::Pod::Parser::AddHeadLevels;
 use Perl6::Pod::To::DocBook::ProcessHeads;
+use Perl6::Pod::Parser::ListLevels;
+use Perl6::Pod::Parser::Doformatted;
+use Perl6::Pod::Parser::Doallow;
 use XML::ExtOn('create_pipe');
 use base qw/Perl6::Pod::To::XML/;
 use constant POD_URI => 'http://perlcabal.org/syn/S26.html';
@@ -49,8 +74,15 @@ use Data::Dumper;
 sub new {
     my $class = shift;
     my $self  = $class->SUPER::new(@_);
-    $self->{out_put} = create_pipe( 'Perl6::Pod::To::DocBook::ProcessHeads', $self->{out_put});
-    return create_pipe( 'Perl6::Pod::Parser::AddHeadLevels','Test::Filter', $self );
+    $self->{out_put} =
+      create_pipe( 'Perl6::Pod::To::DocBook::ProcessHeads', $self->{out_put} );
+    return create_pipe(
+        'Perl6::Pod::Parser::Doformatted',
+        'Perl6::Pod::Parser::Doallow',
+        'Perl6::Pod::Parser::ListLevels',
+        'Perl6::Pod::Parser::AddHeadLevels',
+        'Test::FilterDocBook', $self
+    );
 }
 
 sub start_document {
@@ -70,7 +102,6 @@ sub start_document {
         }
         my $root = $out->mk_element( $self->{doctype} || 'chapter' );
         $out->start_element($root);
-        #$out->on_start_prefix_mapping( pod => POD_URI );
     }
 }
 
@@ -95,12 +126,6 @@ sub _make_xml_element {
         }
         $out_attr->{$key} = $xml_str;
     }
-
-    #%{ $out_elem->attrs_by_name } = %{ $elem->get_attr };
-    #$out_elem->attrs_by_ns_uri(POD_URI)->{type} = $e_type;
-
-    # add use="SOME::Test::Element"
-    #    if ( exists $elem->current_context->use->{ $e_type } )
     return $out_elem;
 }
 
@@ -158,7 +183,7 @@ sub on_end_block {
     return $el unless $el->isa('Perl6::Pod::Block');
     my $content = exists $el->{_CONTENT_} ? $el->{_CONTENT_} : undef;
     my $data = $self->__handle_export( $el, @$content );
-    my $cel = $self->current_element;
+    my $cel = $self->current_root_element;
     if ($cel) {
         push @{ $cel->{_CONTENT_} }, ref($data) eq 'ARRAY' ? @$data : $data;
         return;
@@ -170,12 +195,69 @@ sub on_end_block {
     return $el;
 }
 
+sub export_block_item {
+    my ( $self, $el, @p ) = @_;
+    my $elem =
+      $self->mk_element('listitem')
+      ->add_content(
+         $self->mk_element('para')->add_content( $self->_make_events(@p) ) );
+#        $self->mk_element('para')->add_content( $self->mk_characters(@p) ) );
 
 
+    #add PDO attr for use in export_block_itemlist
+    #for varlistentry
+    $elem->{POD} = $el->get_attr;
+    return $elem;
+}
+
+sub export_block_itemlist {
+    my ( $self, $el, @p ) = @_;
+    my $attr = $el->attrs_by_name;
+    my ( $list_name, $items_name ) = @{
+        {
+            ordered    => [ 'orderedlist',  'listitem' ],
+            unordered  => [ 'itemizedlist', 'listitem' ],
+            definition => [ 'variablelist', 'listitem' ]
+        }->{ $attr->{listtype} }
+      };
+    my $list = $self->mk_element($list_name);
+    foreach (@p) {
+        next unless ref $_;
+        my $lname = $_->local_name;
+        if ( $lname eq 'listitem' ) {
+            $_->local_name($items_name);
+
+            #if variable list, then add varlistentry
+            if ( $list->local_name eq 'variablelist' ) {
+                my ($term) =
+                  ref( $_->{POD}->{term} )
+                  ? @{ $_->{POD}->{term} }
+                  : ( $_->{POD}->{term} );
+
+                my $var_entry =
+                  $self->mk_element('varlistentry')
+                  ->add_content( $self->mk_element('term')
+                      ->add_content( $self->mk_characters($term) ) );
+                $var_entry->add_content($_);
+                $list->add_content($var_entry);
+
+            }
+            else {
+                $list->add_content($_);
+            }
+
+        }
+    }
+
+    return $list;
+}
+
 sub export_block_NAME {
-    my ($self, $el , $text) = @_;
-    return $self->mk_element('title')->add_content( $self->mk_characters( $text))
+    my ( $self, $el, @in ) = @_;
+    return $self->mk_element('title')
+      ->add_content( $self->_make_events(@in) ) ;
 }
+
 1;
 __END__
 
@@ -190,7 +272,7 @@ Zahatski Aliaksandr, <zag@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2009 by Zahatski Aliaksandr
+Copyright (C) 2009-2010 by Zahatski Aliaksandr
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.8 or,
